@@ -38,6 +38,12 @@ G_BEGIN_DECLS
 GST_DEBUG_CATEGORY_EXTERN (check_debug);
 #define GST_CAT_DEFAULT check_debug
 
+#define __CHECK_VERSION_LATER_THAN(major,minor,micro) \
+    (CHECK_MAJOR_VERSION > major || \
+     (CHECK_MAJOR_VERSION == (major) && CHECK_MINOR_VERSION > (minor)) || \
+     (CHECK_MAJOR_VERSION == (major) && CHECK_MINOR_VERSION == (minor) && \
+      CHECK_MICRO_VERSION > (micro)))
+
 /* logging function for tests
  * a test uses g_message() to log a debug line
  * a gst unit test can be run with GST_TEST_DEBUG env var set to see the
@@ -73,6 +79,11 @@ GstElement *gst_check_setup_element (const gchar * factory);
 void gst_check_teardown_element (GstElement * element);
 GstPad *gst_check_setup_src_pad (GstElement * element,
     GstStaticPadTemplate * template, GstCaps * caps);
+GstPad * gst_check_setup_src_pad_by_name (GstElement * element,
+          GstStaticPadTemplate * template, gchar *name);
+GstPad * gst_check_setup_sink_pad_by_name (GstElement * element, 
+          GstStaticPadTemplate * template, gchar *name);
+void gst_check_teardown_pad_by_name (GstElement * element, gchar *name);
 void gst_check_teardown_src_pad (GstElement * element);
 void gst_check_drop_buffers ();
 void gst_check_caps_equal (GstCaps * caps1, GstCaps * caps2);
@@ -103,9 +114,7 @@ gst_check_message_error (msg, GST_MESSAGE_ERROR,		\
  *
  * wrapper for checks END_TEST
  */
-#if CHECK_MAJOR_VERSION > 0 || \
-    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION > 9) || \
-    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION == 9 && CHECK_MICRO_VERSION > 3)
+#if __CHECK_VERSION_LATER_THAN(0,9,3)
 #define GST_START_TEST(__testname) \
 static void __testname (int __i__)\
 {\
@@ -259,9 +268,11 @@ MAIN_SYNCHRONIZE();
 G_STMT_START {				\
   _gst_check_threads_running = TRUE;	\
 					\
-  mutex = g_mutex_new ();		\
-  start_cond = g_cond_new ();		\
-  sync_cond = g_cond_new ();		\
+  if (mutex == NULL) {			\
+    mutex = g_mutex_new ();		\
+    start_cond = g_cond_new ();		\
+    sync_cond = g_cond_new ();		\
+  }					\
 } G_STMT_END;
 
 #define MAIN_START_THREAD_FUNCTIONS(count, function, data)	\
@@ -302,6 +313,8 @@ G_STMT_START {							\
   /* join all threads */					\
   GST_DEBUG ("MAIN: joining");					\
   g_list_foreach (thread_list, (GFunc) g_thread_join, NULL);	\
+  g_list_free (thread_list);					\
+  thread_list = NULL;						\
   GST_DEBUG ("MAIN: joined");					\
 } G_STMT_END;
 
@@ -402,8 +415,9 @@ fail_unless (gst_element_set_state (element,			\
 #define GST_CHECK_MAIN(name)					\
 int main (int argc, char **argv)				\
 {								\
-  Suite *s = name ## _suite ();					\
+  Suite *s;                                                     \
   gst_check_init (&argc, &argv);				\
+  s = name ## _suite ();					\
   return gst_check_run_suite (s, # name, __FILE__);		\
 }
 
@@ -412,9 +426,16 @@ int main (int argc, char **argv)				\
 
 gboolean _gst_check_run_test_func (const gchar * func_name);
 
-#if CHECK_MAJOR_VERSION > 0 || \
-    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION > 9) || \
-    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION == 9 && CHECK_MICRO_VERSION > 3)
+#if __CHECK_VERSION_LATER_THAN(0,9,6)
+static inline void
+__gst_tcase_add_test (TCase * tc, TFun tf, const char * fname, int signal,
+    int allowed_exit_value, int start, int end)
+{
+  if (_gst_check_run_test_func (fname)) {
+    _tcase_add_test (tc, tf, fname, signal, allowed_exit_value, start, end);
+  }
+}
+#elif __CHECK_VERSION_LATER_THAN(0,9,3)
 static inline void
 __gst_tcase_add_test (TCase * tc, TFun tf, const char * fname, int signal,
     int start, int end)
@@ -435,6 +456,8 @@ __gst_tcase_add_test (TCase * tc, TFun tf, const char * fname, int signal)
 
 #define _tcase_add_test __gst_tcase_add_test
 
+#undef __CHECK_VERSION_LATER_THAN
+
 G_END_DECLS
 
 #endif /* __GST_CHECK_H__ */
